#include "main.h"

extern GLuint	g_Texture[MAX_TEXTURES];		// Speicher fr Texturen

void ChangeToFullScreen()
{
	DEVMODE dmSettings;									

	memset(&dmSettings,0,sizeof(dmSettings));			
	
	if(!EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&dmSettings))
	{
		MessageBox(NULL, "Could Not Enum Display Settings", "Error", MB_OK);
		return;
	}

	dmSettings.dmPelsWidth	= SCREEN_WIDTH;				
	dmSettings.dmPelsHeight	= SCREEN_HEIGHT;			
	
	int result = ChangeDisplaySettings(&dmSettings,CDS_FULLSCREEN);	

	if(result != DISP_CHANGE_SUCCESSFUL)
	{

		MessageBox(NULL, "Display Mode Not Compatible", "Error", MB_OK);
		PostQuitMessage(0);
	}
}

AUX_RGBImageRec *LoadBMP(char *Filename)				
{
        FILE *File=NULL;								
        if (!Filename)									
        {
                return NULL;						
        }
        File=fopen(Filename,"r");				
        if (File)										
        {
			fclose(File);								
			return auxDIBImageLoad(Filename);			
        }
        return NULL;
}

bool LoadGLTextures()									
{
	for(int i=0; i < MAX_TEXTURES;i++)
	{
		
		AUX_RGBImageRec *TextureImage[1];				
		memset(TextureImage,0,sizeof(void *)*1);	

		char file[255];
		strcpy(file,g_TextureFiles[i]);

		if (TextureImage[0]=LoadBMP(file)) 			{
			
			glGenTextures(1, &g_Texture[i]);

			glBindTexture(GL_TEXTURE_2D, g_Texture[i]);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
     }

    if (TextureImage[0])							
		{
			if (TextureImage[0]->data)				
			{
				free(TextureImage[0]->data);		
			}
			free(TextureImage[0]);						
		}

	}
	return true;
}

HWND CreateTheWindow(LPSTR strWindowName, int width, int height, DWORD dwStyle, bool bFullScreen, HINSTANCE hInstance)
{
	HWND hWnd;
	WNDCLASS wndclass;
	
	memset(&wndclass, 0, sizeof(WNDCLASS));				
	wndclass.style = CS_HREDRAW | CS_VREDRAW;			
	wndclass.lpfnWndProc = WinProc;						
	wndclass.hInstance = hInstance;						
	wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);	
	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);		
	wndclass.hbrBackground = (HBRUSH) (COLOR_WINDOW+1);	
	wndclass.lpszClassName = TITEL;

	RegisterClass(&wndclass);							
	
	if(bFullScreen && !dwStyle) 						
	{													
		dwStyle = WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
		ChangeToFullScreen();							
		ShowCursor(FALSE);								
	}
	else if(!dwStyle)									
		dwStyle = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	
	g_hInstance = hInstance;							

	RECT rWindow;
	rWindow.left	= 0;								
	rWindow.right	= width;							
	rWindow.top	    = 0;								
	rWindow.bottom	= height;							

	AdjustWindowRect( &rWindow, dwStyle, false);		

														
	hWnd = CreateWindow(TITEL, strWindowName, dwStyle, 0, 0,
						rWindow.right  - rWindow.left, rWindow.bottom - rWindow.top, 
						NULL, NULL, hInstance, NULL);

	if(!hWnd) return NULL;								

	ShowWindow(hWnd, SW_SHOWNORMAL);			
	UpdateWindow(hWnd);									

	SetFocus(hWnd);										

	return hWnd;
}

bool bSetupPixelFormat(HDC hdc) 
{ 
    PIXELFORMATDESCRIPTOR pfd; 
    int pixelformat; 
 
    pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);			
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER; 
    pfd.dwLayerMask = PFD_MAIN_PLANE;	
    pfd.iPixelType = PFD_TYPE_RGBA;		
    pfd.cColorBits = SCREEN_DEPTH;		
    pfd.cDepthBits = SCREEN_DEPTH;		
    pfd.cAccumBits = 0;								
    pfd.cStencilBits = 0;							
 	
    if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == FALSE ) 
    { 
        MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK); 
        return FALSE; 
    } 
 
    if (SetPixelFormat(hdc, pixelformat, &pfd) == FALSE) 
    { 
        MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK); 
        return FALSE; 
    } 
 
    return TRUE;
}

void ReSize(int width, int height)			
{
	if (height==0)										
	{
		height=1;				// um division durch 0 zu verhindern						
	}

	glViewport(0,0,width,height);	

	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();						
														
								  // FOV		// Ratio								//  near, far clipping plane
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, 0.01f,5000.0f);
		
	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();									
}

void InitOpenGL(int width, int height) 
{  
	g_hDC = GetDC(g_hWnd);						
	if (!bSetupPixelFormat(g_hDC))		
		PostQuitMessage (0);					

	g_hRC = wglCreateContext(g_hDC);	
	wglMakeCurrent(g_hDC, g_hRC);			

	ReSize(width, height);		
}

void Shutdown()
{
	if (g_hRC)											
	{
		if (!wglMakeCurrent(NULL,NULL))	
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		
		if (!wglDeleteContext(g_hRC))		
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		
		g_hRC=NULL;										
	}

	if (g_hDC && !ReleaseDC(g_hWnd,g_hDC))					
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		g_hDC=NULL;										
	}

	if (g_hWnd && !DestroyWindow(g_hWnd))					
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		g_hWnd=NULL;										
	}

	if (g_bFullScreen)										
	{
		// falls wir Fullscreen waren, zurck zum desktop schalten 
		ChangeDisplaySettings(NULL,0);			
		// und den Mousezeiger wieder anzeigen
		ShowCursor(TRUE);								
	}

	UnregisterClass(TITEL, g_hInstance);
	PostQuitMessage (0);								
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hprev, PSTR cmdline, int ishow)
{	
	HWND hWnd;
	
	// user-eingabe: Fullscreen oder windowed mode
	if (MessageBox(NULL,"Run in Fullscreen mode?", "FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
		g_bFullScreen=FALSE;
	
	hWnd = CreateTheWindow(TITEL, SCREEN_WIDTH, SCREEN_HEIGHT, 0, g_bFullScreen, hInstance);
	
	if(hWnd == NULL) return true;
	
	Init(hWnd);													
	
	return MainLoop();						
}
