#include "sys_timer.h"
#include <windows.h>
#include <mmsystem.h>

CTimer::CTimer() : 
		m_pfnUpdate(0),
		m_fBaseTime(0.0f),
		m_fLastTime(0.0f),
		m_fSecsPerTick(0.0f),
		m_fCurrentTime(0.0f),
		m_fFrameTime(0.0f),
		m_dTimerStart(0),
		m_dCurTime(0)
{
}

CTimer::~CTimer()
{	m_pfnUpdate = 0;
}

//- Initialisierung
void CTimer::Init()
{
	_int64	dTicksPerSec =0;

	if (QueryPerformanceFrequency(reinterpret_cast<LARGE_INTEGER *>(&dTicksPerSec)))
	{ 
		//- Der Performanc Counter ist verfgbar und wird  benutzt.
		QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER *>(&m_dTimerStart));
		m_fSecsPerTick = (1.0f)/(static_cast<float>(dTicksPerSec));
		m_pfnUpdate = &CTimer::GetPerformanceCounterTime;
	}
	else
	{ 
		//- Falls der PC nicht verfgbar ist, wird der MM-timer verwendet.
		m_dTimerStart = timeGetTime();
		m_fSecsPerTick = 1.0f/1000.0f;
		m_pfnUpdate = &CTimer::GetMMTime;
	}
}

//- Update, einmal pro Frame.
void CTimer::Update()
{
	m_fCurrentTime =  (this->*m_pfnUpdate)() - m_fBaseTime;
	m_fFrameTime = m_fCurrentTime - m_fLastTime;
	m_fLastTime = m_fCurrentTime;
}

//- Reset, Zeit wird auf 0.0 gestellt.
void CTimer::Reset()
{
	m_fBaseTime =  (this->*m_pfnUpdate)();
	m_fLastTime = m_fCurrentTime = m_fFrameTime = 0.0;
}

//- Update Funktionen des PC und des MM-Timers.
float CTimer::GetPerformanceCounterTime()
{
	QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER *>(&m_dCurTime));
	return (m_fSecsPerTick * (m_dCurTime - m_dTimerStart));
}

float CTimer::GetMMTime()
{	return (m_fSecsPerTick * (timeGetTime()- m_dTimerStart));
}